/***************************************************************************
 *   Copyright (C) 2004 by Georges A.K.                                    *
 *   georgesak@spymac.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef _KBEAMDEVICE_H_
#define _KBEAMDEVICE_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/socket.h>
#include <linux/types.h>
#include <linux/irda.h>

#include <qstring.h>


/** This class contains all the data necessary for a device

 * @short Class representing a device
 * @author Georges A.K. <georgesak@spymac.com>
 */
 
class KBeamDevice
{

public:

	/** Default constructor */
	KBeamDevice();
	/** Constructor */
	KBeamDevice(irda_device_info pInfo);

	/** Returns the address of the device */
	const unsigned int getAddress() const { return mAddress; }
	/** Returns the description of the device */
	const QString getName() const { return mName; }
	/** Returns the hints of the device */
	const unsigned char getHint(const int pNum) const { return mHints[pNum]; }

	/** Overload of the == operator */
	bool operator == (const KBeamDevice &pDevice) const {
		return mAddress == pDevice.mAddress;
	}

	/** These functions check for the different properties */
	bool checkPnp() const { return mHints[0] & HINT_PNP; }
	bool checkPda() const { return mHints[0] & HINT_PDA; }
	bool checkComputer() const { return mHints[0] & HINT_COMPUTER; }
	bool checkPrinter() const { return mHints[0] & HINT_PRINTER; }
	bool checkModem() const { return mHints[0] & HINT_MODEM; }
	bool checkFax() const { return mHints[0] & HINT_FAX; }
	bool checkLan() const { return mHints[0] & HINT_LAN; }
	bool checkExt() const { return mHints[0] & HINT_EXTENSION; }
	bool checkTel() const { return mHints[1] & HINT_TELEPHONY; }
	bool checkFS() const { return mHints[1] & HINT_FILE_SERVER; }
	bool checkComm() const { return mHints[1] & HINT_COMM; }
	bool checkMessage() const { return mHints[1] & HINT_MESSAGE; }
	bool checkHttp() const { return mHints[1] & HINT_HTTP; }
	bool checkObex() const { return mHints[1] & HINT_OBEX; }

private:

	unsigned int mAddress;
	QString mName;
	unsigned char mHints[2];

};

#endif // _KBEAMDEVICE_H_
