/***************************************************************************
 *   Copyright (C) 2004 by Georges A.K.                                    *
 *   georgesak@spymac.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kstandarddirs.h> 
#include <kpixmap.h>
#include <klocale.h>
#include <qlistview.h>

#include "kbeamdevicedlg.h"
#include "kbeamdevicelist.h"


KBeamDeviceDlg::KBeamDeviceDlg(QWidget *pParent) : 
	KDialogBase(pParent, "", false, "", KDialogBase::Close)
{
	mDeviceListView = new QListView(this);

	setMainWidget(mDeviceListView);

	// Disable selection and sorting
	mDeviceListView->setSelectionMode(QListView::NoSelection);
	mDeviceListView->setSorting(-1);

	mDeviceListView->addColumn(i18n("Name"));
	mDeviceListView->setColumnWidth(0, 200);
	mDeviceListView->addColumn(i18n("Type"));
	mDeviceListView->setColumnWidth(1, 100);
	mDeviceListView->addColumn(i18n("Obex"));
	mDeviceListView->setColumnWidth(2, 100);
	mDeviceListView->addColumn(i18n("Address"));
	mDeviceListView->setColumnWidth(3, 200);
}

KBeamDeviceDlg::~KBeamDeviceDlg()
{
	delete mDeviceListView;
}

void KBeamDeviceDlg::slotUpdateDevices(const KBeamDeviceList &pDeviceList)
{
	// Clear list
	mDeviceListView->clear();

	for (int i=0;i<pDeviceList.getNbDevices();i++) {
		const KBeamDevice *lDevice = &pDeviceList.getDevice(i);
		QString lType = i18n("Unknown");
		KPixmap lIcon;

		// Check the device type and icon
		if (lDevice->checkPda()) {
			lType = i18n("PDA");
			lIcon.load(locate("appdata", "icons/pda.png"));
		} else if (lDevice->checkComputer()) {
			lType = i18n("Computer");
			lIcon.load(locate("appdata", "icons/computer.png"));
		} else if (lDevice->checkPrinter()) {
			lType = i18n("Printer");
			lIcon.load(locate("appdata", "icons/printer.png"));
		} else if (lDevice->checkPnp()) {
			lType = i18n("Phone");
			lIcon.load(locate("appdata", "icons/phone.png"));
		}

		// Does the device support the Obex protocol ?
		QString lObex = i18n("Not supported");
		if (lDevice->checkObex()) {
			lObex = i18n("Supported");
		}

		// Create the item for the device and add it to the list view
		QListViewItem *lItem = new QListViewItem(mDeviceListView, 
			lDevice->getName(), lType, lObex, 
			QString::number(lDevice->getAddress()));
		lItem->setPixmap(1, lIcon);
	}
}
