/***************************************************************************
 *   Copyright (C) 2004 by Georges A.K.                                    *
 *   georgesak@spymac.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _KBEAMDEVICELIST_H_
#define _KBEAMDEVICELIST_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/socket.h>
#include <linux/types.h>
#include <linux/irda.h>

#include <qvaluelist.h>

#include "kbeamdevice.h"


/** This class contains the list of all devices in range. It inherits from
    QValueList.

 * @short Contains the list of devices in range
 * @author Georges A.K. <georgesak@spymac.com>
 */
 
class KBeamDeviceList : public QValueList<KBeamDevice>
{

public:

	/** Returns the number of devices in range */
	int getNbDevices() const { return count(); }
	/** Returns the device at an index */
	const KBeamDevice &getDevice(int pNum) const { 
		return (*this)[pNum]; 
	}

	/** Returns true if the device is new */
	bool isDeviceNew(const KBeamDevice &pDevice) const;
	/** Updates the list of devices */
	void updateDevices(const irda_device_list *pList);

};

#endif // _KBEAMDEVICELIST_H_
