/***************************************************************************
 *   Copyright (C) 2004 by Georges A.K.                                    *
 *   georgesak@spymac.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kapplication.h>
#include "kbeamevent.h"


KBeamEvent::KBeamEvent(EventType pType) : 
	QCustomEvent(pType)
{
}

KBeamEvent::KBeamEvent(EventType pType, const QString &pString) : 
	QCustomEvent(pType)
{
	mString = pString;
}

KBeamEvent::KBeamEvent(EventType pType, const KBeamDevice &pDevice) : 
	QCustomEvent(pType)
{
	mDevice = pDevice;
}

void KBeamEvent::sendErrorMsg(const QString &pErrorMsg)
{
	KApplication::postEvent(kapp->mainWidget(), 
		new KBeamEvent(KBeamEvent::ErrorMsg, pErrorMsg));
}

void KBeamEvent::sendDeviceFound(const KBeamDevice &pDevice)
{
	KApplication::postEvent(kapp->mainWidget(), 
		new KBeamEvent(KBeamEvent::DeviceFound, pDevice));
}

void KBeamEvent::sendDeviceLost(const KBeamDevice &pDevice)
{
	KApplication::postEvent(kapp->mainWidget(), 
		new KBeamEvent(KBeamEvent::DeviceLost, pDevice));
}

void KBeamEvent::sendStartReceive(const QString &pFilename)
{
	KApplication::postEvent(kapp->mainWidget(), 
		new KBeamEvent(KBeamEvent::StartReceive, pFilename));
}

void KBeamEvent::sendFileReceived(const QString &pFilename)
{
	KApplication::postEvent(kapp->mainWidget(), 
		new KBeamEvent(KBeamEvent::FileReceived, pFilename));
}

void KBeamEvent::sendReceiveAborted(const QString &pFilename)
{
	KApplication::postEvent(kapp->mainWidget(), 
		new KBeamEvent(KBeamEvent::ReceiveAborted, pFilename));
}

void KBeamEvent::sendStartSend(const QString &pFilename)
{
	KApplication::postEvent(kapp->mainWidget(), 
		new KBeamEvent(KBeamEvent::StartSend, pFilename));
}

void KBeamEvent::sendFileSent(const QString &pFilename)
{
	KApplication::postEvent(kapp->mainWidget(), 
		new KBeamEvent(KBeamEvent::FileSent, pFilename));
}

void KBeamEvent::sendSendAborted(const QString &pFilename)
{
	KApplication::postEvent(kapp->mainWidget(), 
		new KBeamEvent(KBeamEvent::SendAborted, pFilename));
}

void KBeamEvent::sendLinkInterrupted()
{
	KApplication::postEvent(kapp->mainWidget(), 
		new KBeamEvent(KBeamEvent::LinkInterrupted));
}
