/***************************************************************************
 *   Copyright (C) 2004 by Georges A.K.                                    *
 *   georgesak@spymac.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef _KBEAMDEVENT_H_
#define _KBEAMDEVENT_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qevent.h>

#include "kbeamdevice.h"


/** This class inherits from QCustomEvent and takes care of all the 
    asynchronous events between objects (all synchronous events are handled
    by the use of signals

 * @short Class that takes care of all events
 * @author Georges A.K. <georgesak@spymac.com>
 * @version 0.1
 */
 
class KBeamEvent : public QCustomEvent
{

public:

	/** The different events */
	enum EventType {	ErrorMsg        = int(QEvent::User) + 0,
						DeviceFound     = int(QEvent::User) + 1, 
						DeviceLost      = int(QEvent::User) + 2,
						StartReceive    = int(QEvent::User) + 3,
						FileReceived    = int(QEvent::User) + 4,
						ReceiveAborted  = int(QEvent::User) + 5,
						StartSend       = int(QEvent::User) + 6,
						FileSent        = int(QEvent::User) + 7,
						SendAborted     = int(QEvent::User) + 8,
						LinkInterrupted = int(QEvent::User) + 9
	};

	/** Default constructor */
	KBeamEvent(EventType pType);
	/** Constructor */
	KBeamEvent(EventType pType, const QString &pString);
	/** Constructor */
	KBeamEvent(EventType pType, const KBeamDevice &pDevice);

	/** Returns the error message */
	const QString &getErrorMsg()const { return mString; }
	/** Returns the device */
	const KBeamDevice &getDevice()const { return mDevice; }
	/** Returns the filename */
	const QString &getFilename()const { return mString; }

	/** Sends event ErrorMsg */
	static void sendErrorMsg(const QString &pErrorMsg);
	/** Sends event DeviceFound */

	static void sendDeviceFound(const KBeamDevice &pDevice);
	/** Sends event DeviceLost */
	static void sendDeviceLost(const KBeamDevice &pDevice);
	/** Sends event StartReceive */

	static void sendStartReceive(const QString &pFilename);
	/** Sends event FileReceived */
	static void sendFileReceived(const QString &pFilename);
	/** Sends event ReceiveAborted */
	static void sendReceiveAborted(const QString &pFilename);

	/** Sends event StartSend */
	static void sendStartSend(const QString &pFilename);
	/** Sends event FileSent */
	static void sendFileSent(const QString &pFilename);
	/** Sends event SendAborted */
	static void sendSendAborted(const QString &pFilename);

	/** Sends event LinkInterrupted */
	static void sendLinkInterrupted();

private:

	KBeamDevice mDevice;
	QString mString;

};

#endif // _KBEAMDEVENT_H_
