/***************************************************************************
 *   Copyright (C) 2004 by Georges A.K.                                    *
 *   georgesak@spymac.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _KBEAMOBEX_H_
#define _KBEAMOBEX_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qobject.h>
#include <qguardedptr.h>
extern "C" {
#include <openobex/obex.h>
}

class QSocketNotifier;
class KBeamObexObject;


/** This class wraps the structure obex_t and allows us to use all of the
	related functions in an Object Oriented way. A special thanks to the 
	team behind the project KDE Bluetooth as this code a lot inspired from
	theirs

 * @short A wrapper class for obex_t
 * @author Georges A.K. <georgesak@spymac.com>
 */
 
class KBeamObex : public QObject
{
	Q_OBJECT

public:

	/** Default constructor */
	KBeamObex();
	/** Constructor */
	KBeamObex(obex_t *pHandle);
	/** Destructor */
	~KBeamObex();

	/** Return the Obex handle */
	obex_t *getHandle() const { return mHandle; }

	/** Register the server and start listening for incoming requests */
	void registerServer();
	/** Connect to the remote device */
	bool transportConnect();
	/** Disconnect from the remote device */
	void transportDisconnect();

	/** Creates the socket notifier */
	void createNotifier();
	/** Destroys the socket notifier */
	void deleteNotifier();
	/** Pauses the socket notifier */
	void pauseNotifier();
	/** Resumesthe socket notifier */
	void resumeNotifier();

	/** Sends a request to the remote device*/
	void request(KBeamObexObject &pObject);
	/** Cancels a request */
	void cancelRequest(int pNice);

	/** Destroys the Obex handle */
	void cleanup();

	/** Accepts the incoming connection and returns the handle */
	obex_t *accept() const;

	/** The different commands */
	enum Command {	Connect = OBEX_CMD_CONNECT,
					Disconnect = OBEX_CMD_DISCONNECT,
					Put = OBEX_CMD_PUT,
					Get = OBEX_CMD_GET,
					SetPath = OBEX_CMD_SETPATH,
					Abort = OBEX_CMD_ABORT };

	/** The different responses */
	enum Response {	Continue = OBEX_RSP_CONTINUE,
					SwitchPro = OBEX_RSP_SWITCH_PRO,
					Success = OBEX_RSP_SUCCESS,
					Created = OBEX_RSP_CREATED,
					Accepted = OBEX_RSP_ACCEPTED,
					NoContent = OBEX_RSP_NO_CONTENT,
					BadRequest = OBEX_RSP_BAD_REQUEST,
					Unauthorized = OBEX_RSP_UNAUTHORIZED,
					PaymentRequired = OBEX_RSP_PAYMENT_REQUIRED,
					Forbidden = OBEX_RSP_FORBIDDEN,
					NotFound = OBEX_RSP_NOT_FOUND,
					MethodNotAllowed = OBEX_RSP_METHOD_NOT_ALLOWED,
					Conflict = OBEX_RSP_CONFLICT,
					InternalServerError = OBEX_RSP_INTERNAL_SERVER_ERROR,
					NotImplemented = OBEX_RSP_NOT_IMPLEMENTED,
					DatabaseFull = OBEX_RSP_DATABASE_FULL,
					DatabaseLocked = OBEX_RSP_DATABASE_LOCKED };

	/** Converts a response into a QString*/
	static QString responseToQString(Response pRsp);
	/** Converts a QString into Unicode */
	static QByteArray* qStringToUnicode(const QString &pArray);
	/** Converts an Obex string into a QString */
	static QString obexstringToQString(unsigned short* pBuf);

signals:
	void signalProgressMade(KBeamObexObject &pObject);
	void signalRequestAborted(KBeamObexObject &pObject);
	void signalRequestHint(KBeamObexObject &pObject, KBeamObex::Command pCmd);
	void signalRequestDone(KBeamObexObject &pObject, KBeamObex::Command pCmd,
		KBeamObex::Response pRsp);
	void signalRequest(KBeamObexObject &pObject, KBeamObex::Command pCmd);
	void signalLinkError();
	void signalStreamEmpty(KBeamObexObject &pObject);
	void signalStreamAvailble(KBeamObexObject &pObject);
	void signalAcceptHint(KBeamObex &pObex, KBeamObexObject &pObject);
	void transportError();
	void transportConnected();

private:

	/** This is the call back function to handle an event */
	void handleEvent(obex_object_t *pObject, int pMode, int pEvent,
		int pObexCmd, int pObexRsp);

	/** This is the actual call back function to handle an event. It will just
		find out the object and call handleEvent(...) */
	static void handleEventStatic(obex_t *pHandle, obex_object_t *pObject,
		int pMode, int pEvent, int pObexCmd, int pObexRsp);

private slots:

	/** This slot is called by the socket notifier whenever there's data to 
		be read */
	void slotHandleInput(int);

private:

	obex_t *mHandle;
	QGuardedPtr<QSocketNotifier> mNotifier;
	bool mConnected;

};

#endif  //_KBEAMOBEX_H_
