/***************************************************************************
 *   Copyright (C) 2004 by Georges A.K.                                    *
 *   georgesak@spymac.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _KBEAMOBEXHEADER_H_
#define _KBEAMOBEXHEADER_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

extern "C" {
#include <openobex/obex.h>
}

class KBeamObexObject;


/** This class wraps the structure obex_headerdata_t and allows us to use all 
	of the related functions in an Object Oriented way. A special thanks to the
	team behind the project KDE Bluetooth as this code a lot inspired from
	theirs

 * @short A wrapper class for obex_headerdata_t
 * @author Georges A.K. <georgesak@spymac.com>
 */
 
class KBeamObexHeader
{

public:
	/** Header types */
	enum Type { 	None = 0,
					Length = OBEX_HDR_LENGTH,
					Name = OBEX_HDR_NAME,
					Body = OBEX_HDR_BODY,
					Who = OBEX_HDR_WHO,
					MimeType = OBEX_HDR_TYPE };

	/** Constructor */
	KBeamObexHeader(Type pType, uint32_t pBq4, uint32_t pSize);
	/** Constructor */
	KBeamObexHeader(Type pType, uint8_t pBq1, uint32_t pSize);
	/** Constructor */
	KBeamObexHeader(Type pType, const uint8_t *pBs, uint32_t pSize);
	/** Detructor */
	~KBeamObexHeader();

	/** Returns the header type*/
	Type getType();
	/** Returns data*/
	void data(uint32_t &pBq4, uint32_t &pSize);
	/** Returns data*/
	void data(uint8_t &pBq1, uint32_t &pSize);
	/** Returns data*/
	void data(const uint8_t *&pBs, uint32_t &pSize);

private:

	/** Private constructor (used by friend class KBeamObexObject */
	KBeamObexHeader(uint8_t pType, obex_headerdata_t pHd, uint32_t pSize);

	uint8_t mHi;
	obex_headerdata_t mHv;
	uint32_t mHvSize;

friend class KBeamObexObject;

};

#endif  //_KBEAMOBEXHEADER_H_
