/***************************************************************************
 *   Copyright (C) 2004 by Georges A.K.                                    *
 *   georgesak@spymac.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kbeamobexobject.h"
#include "kbeamobex.h"
#include "kbeamobexheader.h"


KBeamObexObject::KBeamObexObject(KBeamObex &pObex, KBeamObex::Command pCmd)
{
	mHandle = pObex.getHandle();
	mObject = OBEX_ObjectNew(mHandle, pCmd);
}

KBeamObexObject::KBeamObexObject(obex_t *pHandle, obex_object_t *pObject)
{
	mHandle = pHandle;
	mObject = pObject;
}

KBeamObexObject::~KBeamObexObject()
{
}

void KBeamObexObject::addHeader(KBeamObexHeader &pHeader, addHeaderFlag pFlags)
{
	OBEX_ObjectAddHeader(mHandle, mObject, pHeader.mHi, pHeader.mHv,
		pHeader.mHvSize, pFlags);
}

KBeamObexHeader* KBeamObexObject::getNextHeader()
{
	uint8_t lHi;
	obex_headerdata_t lHv;
	uint32_t lHvSize;

	if (OBEX_ObjectGetNextHeader(mHandle, mObject, &lHi, &lHv, &lHvSize) != 0)
	{
		KBeamObexHeader *lHeader = new KBeamObexHeader (lHi, lHv, lHvSize);
		return lHeader;
	}

	return 0;
}

int KBeamObexObject::readStream(QByteArray &pBuffer)
{
	const uint8_t *lBuf;
	int lSize;

	lSize = OBEX_ObjectReadStream (mHandle, mObject, &lBuf);
	pBuffer.duplicate((char*)lBuf, lSize);

	return lSize;
}

bool KBeamObexObject::enableReadStream()
{
	return OBEX_ObjectReadStream(mHandle, mObject, NULL) >= 0;
}

void KBeamObexObject::setResponse(KBeamObex::Response pRsp,
	KBeamObex::Response pLastRsp)
{
	OBEX_ObjectSetRsp (mObject, int(pRsp), int(pLastRsp));
}
