/***************************************************************************
 *   Copyright (C) 2004 by Georges A.K.                                    *
 *   georgesak@spymac.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _KBEAMOBEXOBJECT_H_
#define _KBEAMOBEXOBJECT_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "kbeamobex.h"

class KBeamObexHeader;


/** This class wraps the structure obex_object_t and allows us to use all 
	of the related functions in an Object Oriented way. A special thanks to the
	team behind the project KDE Bluetooth as this code a lot inspired from
	theirs

 * @short A wrapper class for obex_object_t
 * @author Georges A.K. <georgesak@spymac.com>
 */
 
class KBeamObexObject
{

public:

	/** Constructor */
	KBeamObexObject(KBeamObex &pObex, KBeamObex::Command pCmd);
	/** Destructor */
	~KBeamObexObject();

	/** Types used when adding headers */
	enum addHeaderFlag {	None = 0,
							FitOnePacket = OBEX_FL_FIT_ONE_PACKET,
							StreamStart = OBEX_FL_STREAM_START,
							StreamData = OBEX_FL_STREAM_DATA,
							StreamEnd = OBEX_FL_STREAM_DATAEND };

	/** Adds a header to the object request */
	void addHeader(KBeamObexHeader &pHeader, addHeaderFlag pFlag);
	/** Returns the next header in object */
	KBeamObexHeader* getNextHeader();
	/** Returns the stream */
	int readStream(QByteArray& pBuffer);
	/** Enables the send to be streamed */
	bool enableReadStream();
	/** Returns a response for a command */
	void setResponse(KBeamObex::Response pRsp, KBeamObex::Response pLastRsp);

protected:

	/** Private constructor used by friend class KBeamObex */
	KBeamObexObject(obex_t *pHandle, obex_object_t *pObject);

private :

	obex_t *mHandle;
	obex_object_t *mObject;

friend class KBeamObex;

};

#endif  //_KBEAMOBEXOBJECT_H_
