/***************************************************************************
 *   Copyright (C) 2004 by Georges A.K.                                    *
 *   georgesak@spymac.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kmessagebox.h>
#include <klocale.h>
#include <qframe.h>
#include <qcheckbox.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

#include "kbeamprefsdlg.h"


KBeamPrefsDlg::KBeamPrefsDlg()
{
	const int lWidth = 500;
	const int lHeight = 200;

	setMinimumSize(lWidth, lHeight);

	mMainFrame = new QFrame(this);
	mMainFrame->setGeometry(0, 0, lWidth, lHeight);
	mMainFrame->setFrameShape(QFrame::Box);
	mMainFrame->setFrameShadow(QFrame::Raised);

	mAlwaysVisible = new QCheckBox(mMainFrame, "kcfg_AlwaysVisible");
	mAlwaysVisible->setText(
		i18n("Make icon visible even when there are no connections"));
	mAlwaysVisible->setGeometry(10, 10, lWidth-20, 25);
	connect(mAlwaysVisible, SIGNAL(clicked()), this,
		SLOT(slotAlwaysShowIconClicked()));

	mAutoStart = new QCheckBox(mMainFrame, "kcfg_AutoStart");
	mAutoStart->setText(i18n("Automatically launch KBeam when KDE starts"));
	mAutoStart->setGeometry(10, 40, lWidth-20, 25);

	// Set the tooltips and the whatsthis
	addTips();
}

KBeamPrefsDlg::~KBeamPrefsDlg()
{
	delete mMainFrame;
	delete mAlwaysVisible;
	delete mAutoStart;
}

void KBeamPrefsDlg::addTips()
{
	QWhatsThis::add(mAlwaysVisible, i18n("If selected, the KBeam icon will \
always be shown. Otherwise, it will be hidden when there are no devices in \
range and will only appear when devices are connected. You can always \
make the icon reappear by rerunning KBeam."));
	QWhatsThis::add(mAutoStart, i18n("If selected, KBeam will automatically\
 start whenever you log into KDE."));
}

void KBeamPrefsDlg::slotAlwaysShowIconClicked() {
	if (!mAlwaysVisible->isOn()) {
		KMessageBox::information(this,
			i18n("To make the icon reappear, run kbeam again."),
			i18n("Information"), 
			"Always Show Icon");
	}
}
