/***************************************************************************
 *   Copyright (C) 2004 by Georges A.K.                                    *
 *   georgesak@spymac.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kbeamreceive.h"
#include "kbeamobexobject.h"
#include "kbeamobexheader.h"
#include "kbeamconfig.h"
#include "kbeamevent.h"

#include <kmessagebox.h>
#include <qprogressbar.h>
#include <qfile.h>


KBeamReceive::KBeamReceive() :
	KProgressDialog(NULL, NULL, "KBeam", "Incoming File Transfer")
{
	showCancelButton(true);
	disableResize();
	setAutoClose(false);
	progressBar()->setMinimumSize(300, 25);
	hide();
}

KBeamReceive::~KBeamReceive()
{
	if (mObex) {
		delete mObex;
	}
}


void KBeamReceive::receive(const KBeamObex &pObex)
{
	mObex = new KBeamObex(pObex.accept());

	connect(mObex, SIGNAL(signalRequestHint(KBeamObexObject&, 
		KBeamObex::Command )), this, SLOT(slotRequestHint(KBeamObexObject&, 
		KBeamObex::Command )));
	connect(mObex, SIGNAL(signalRequest(KBeamObexObject&, 
		KBeamObex::Command )), this, SLOT(slotRequest(KBeamObexObject&, 
		KBeamObex::Command )));
	connect(mObex, SIGNAL(signalStreamAvailble(KBeamObexObject& )), this,
		SLOT(slotStreamAvailble(KBeamObexObject& )));
	connect(mObex, SIGNAL(signalRequestDone(KBeamObexObject&, 
		KBeamObex::Command, KBeamObex::Response )), this,
		SLOT(slotRequestDone(KBeamObexObject&, KBeamObex::Command, 
		KBeamObex::Response )));
	connect(mObex, SIGNAL(signalRequestAborted(KBeamObexObject &)), this,
		SLOT(slotRequestAborted(KBeamObexObject &)));
	connect(mObex, SIGNAL(signalLinkError()), this,
		SLOT(slotLinkError()));

	connect(&mLinkTimer, SIGNAL(timeout()), this, SLOT(slotLinkTimeout()));

	mFirstStream = true;
	mFileSize = -1;

	if (KBeamConfig::self()->showReceiveDlg()) {
		show();
	} else {
		hide();
	}
}

void KBeamReceive::slotRequestHint(KBeamObexObject &pObject, 
	KBeamObex::Command pCmd)
{
	switch (pCmd) {
	case KBeamObex::Connect:
	case KBeamObex::Disconnect:
		pObject.setResponse(KBeamObex::Continue, KBeamObex::Success);
		break;
	case KBeamObex::Put:
		pObject.enableReadStream();
		lObjectData.open(IO_WriteOnly);
		pObject.setResponse(KBeamObex::Continue, KBeamObex::Success);
		break;
	default:
		pObject.setResponse(KBeamObex::NotImplemented, 
			KBeamObex::NotImplemented);
		break;
	}
}

void KBeamReceive::slotRequest(KBeamObexObject &pObject, 
	KBeamObex::Command pCmd)
{
	switch (pCmd) {
	case KBeamObex::Connect:
	case KBeamObex::Disconnect:
	case KBeamObex::Put:
		pObject.setResponse(KBeamObex::Success, KBeamObex::Success);
		break;
	default:
		pObject.setResponse (KBeamObex::BadRequest, KBeamObex::BadRequest);
		break;
	}
}

void KBeamReceive::parseHeader(KBeamObexObject &pObject)
{
	KBeamObexHeader *lHeader;
	const uint8_t *lBs;
	uint32_t lBq4;
	uint32_t lSize;
	QByteArray lBuffer;

	while ((lHeader = pObject.getNextHeader ()) != 0) {
		switch (lHeader->getType()) {
		case KBeamObexHeader::Who:
			lHeader->data(lBs, lSize);
			mFilename = KBeamObex::obexstringToQString((unsigned short*)lBs);
			break;
		case KBeamObexHeader::Name:
			lHeader->data(lBs, lSize);
			mFilename = KBeamObex::obexstringToQString((unsigned short*)lBs);
			break;
		case KBeamObexHeader::MimeType:
			lHeader->data(lBs, lSize);
//			mimeType = Obex::obexstringToQString((unsigned short*)bs);
			break;
		case KBeamObexHeader::Length:
			lHeader->data(lBq4, lSize);
			mFileSize = lBq4;
			break;
		default:
			break;
		}
	}
}

void KBeamReceive::slotStreamAvailble(KBeamObexObject &pObject)
{
	int lSize;

	if (isShown() && !KBeamConfig::self()->showReceiveDlg()) {
		hide();
	}

	// Stop the link timer
	mLinkTimer.stop();
	lSize = pObject.readStream(mStream);
	lObjectData.writeBlock(mStream);
	if (mFirstStream) {
		mFirstStream = false;
		mCurrentSize = 0;
		parseHeader(pObject);
		// Send event that we're receiving file
		KBeamEvent::sendStartReceive(mFilename);
		if (mFileSize > -1) {
				progressBar()->setTotalSteps(mFileSize);
			setLabel(QString("Receiving file ") + mFilename + 
				QString("  (") + QString::number(mFileSize) + " bytes)");
		} else {
			progressBar()->setEnabled(false);
			setLabel(QString("Receiving file ") + mFilename + 
				QString("  (unknown size)"));
		}
	}
	mCurrentSize += lSize;
	if (mFileSize > -1) {
		progressBar()->setProgress(mCurrentSize);
 		setCaption(QString::number(mCurrentSize*100/mFileSize) + QString("%"));
	} else {
		setCaption("?? %");
	}

	// Start the link timer
	mLinkTimer.start(2000, true);

}

void KBeamReceive::slotRequestDone(KBeamObexObject &pObject, 
	KBeamObex::Command pCmd, KBeamObex::Response /*pRsp*/)
{
	switch (pCmd) {
	case KBeamObex::Put:
		pObject.setResponse(KBeamObex::Success, KBeamObex::Success);
		saveFile();
		break;
	case KBeamObex::Disconnect:
		pObject.setResponse(KBeamObex::Success, KBeamObex::Success);
		done();
		break;
	default:
		pObject.setResponse(KBeamObex::BadRequest, KBeamObex::BadRequest);
		break;
	}
}

void KBeamReceive::slotLinkError()
{
	aborted("File transfer was aborted because the link was broken");
}

void KBeamReceive::slotRequestAborted(KBeamObexObject &/*pObject*/)
{
	aborted("File transfer was aborted by remote device");
}

void KBeamReceive::saveFile()
{
	QString lFilename = KBeamConfig::receivedPath();
	if (lFilename[lFilename.length()-1] != '/') {
		lFilename += QString("/");
	}
	lFilename += mFilename;

	QFile lFile(lFilename);

	if (lFile.open(IO_WriteOnly)) {
		lFile.writeBlock(lObjectData.buffer());
		lFile.close();
	} else {
		QString lErrorMsg = "There was a problem saving the file : \n";
		lErrorMsg += lFilename;
		lErrorMsg += "\nPlease check that you have write permission !";
		KBeamEvent::sendErrorMsg(lErrorMsg);
	}
}

void KBeamReceive::closeEvent(QCloseEvent */*pEvent*/)
{
	slotCancel();
}

void KBeamReceive::slotCancel()
{
	delete this;
}

void KBeamReceive::slotLinkTimeout()
{
	KBeamEvent::sendLinkInterrupted();
}

void KBeamReceive::done()
{
	mLinkTimer.stop();
	mObex->pauseNotifier();
	setCaption(QString("Done"));
	setButtonText(QString("&Close"));
	progressBar()->setEnabled(true);
	progressBar()->setProgress(mFileSize);
	KBeamEvent::sendFileReceived(mFilename);
	if (isHidden() || KBeamConfig::self()->autoCloseReceiveDlg()) {
		QTimer::singleShot(1000, this, SLOT (slotCancel()));
	}
}

void KBeamReceive::aborted(const QString &pReason)
{
	mLinkTimer.stop();
	mObex->pauseNotifier();
	setCaption(QString("Aborted"));
	progressBar()->setEnabled(false);
	setLabel(pReason);
	setButtonText(QString("&Close"));
	KBeamEvent::sendReceiveAborted(mFilename);
	if (isHidden()) {
		QTimer::singleShot(1000, this, SLOT (slotCancel()));
	}
}

