/***************************************************************************
 *   Copyright (C) 2004 by Georges A.K.                                    *
 *   georgesak@spymac.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _KBEAMRECEIVE_H_
#define _KBEAMRECEIVE_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kprogress.h>
#include <qguardedptr.h>
#include <qbuffer.h>
#include <qcstring.h>
#include <qtimer.h>
#include <qdatetime.h>

#include "kbeamobex.h"


/**
 * @short System calls
 * @author Georges A.K. <georgesak@spymac.com>
 */
class KBeamReceive : public KProgressDialog
{
	Q_OBJECT

public:
	KBeamReceive();
	~KBeamReceive();

	void receive(const KBeamObex &pObex);

private slots:
	
	void slotRequestHint(KBeamObexObject &pObject, KBeamObex::Command pCmd);
	void slotRequest(KBeamObexObject &pObject, KBeamObex::Command pCmd);
	void slotStreamAvailble(KBeamObexObject &pObject);
	void slotRequestDone(KBeamObexObject &pObject, KBeamObex::Command pCmd,
		KBeamObex::Response pRsp);
	void slotLinkError();
	void slotRequestAborted(KBeamObexObject &pObject);
	void slotLinkTimeout();

private:

	void parseHeader(KBeamObexObject &pObject);
	void saveFile();

	void closeEvent(QCloseEvent *pEvent);
	void slotCancel();

	void done();
	void aborted(const QString &pReason);

private:

	QGuardedPtr<KBeamObex> mObex;

	QString mFilename;
	bool mFirstStream;
	int mFileSize;
	QByteArray mStream;
	int mCurrentSize;
	QBuffer lObjectData;
	QTimer mLinkTimer;

};

#endif // _KBEAMRECEIVE_H_
