/***************************************************************************
 *   Copyright (C) 2004 by Georges A.K.                                    *
 *   georgesak@spymac.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kfiledialog.h>
#include <klocale.h>
#include <qframe.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

#include "kbeamreceiveprefsdlg.h"
#include "kbeamconfig.h"


KBeamReceivePrefsDlg::KBeamReceivePrefsDlg()
{
	const int lWidth = 500;
	const int lHeight = 200;

	setMinimumSize(lWidth, lHeight);

	mMainFrame = new QFrame(this);
	mMainFrame->setGeometry(0, 0, lWidth, lHeight);
	mMainFrame->setFrameShape(QFrame::Box);
	mMainFrame->setFrameShadow(QFrame::Raised);

	mShowDlg = new QCheckBox(mMainFrame, "kcfg_ShowReceiveDlg");
	mShowDlg->setText(
		i18n("Show progress dialog when receiving"));
	mShowDlg->setGeometry(10, 10, lWidth-20, 25);
	connect(mShowDlg, SIGNAL(toggled(bool)), this, SLOT(slotShowDlg(bool)));

	mAutoClose = new QCheckBox(mMainFrame, "kcfg_AutoCloseReceiveDlg");
	mAutoClose->setText(
		i18n("Automatically close receive dialog on success"));
	mAutoClose->setGeometry(10, 40, lWidth-20, 25);
	if (!KBeamConfig::self()->showReceiveDlg()) {
		mAutoClose->setEnabled(false);
	}

	mReceivedPathLabel = new QLabel(mMainFrame);
	mReceivedPathLabel->setGeometry(10, 80, lWidth-20, 25);
	mReceivedPathLabel->setText(i18n("Path where received files will be saved :"));

	mReceivedPath = new QLineEdit(mMainFrame, "kcfg_ReceivedPath");
	mReceivedPath->setGeometry(10, 110, lWidth-100, 25);

	mBrowseButton = new QPushButton(mMainFrame);
	mBrowseButton->setGeometry(lWidth-80, 110, 50, 25);
	mBrowseButton->setText(i18n("Browse"));
	connect(mBrowseButton, SIGNAL(clicked()), this, SLOT(slotBrowse()));

	// Set the tooltips and the whatsthis
	addTips();
}

KBeamReceivePrefsDlg::~KBeamReceivePrefsDlg()
{
    delete mMainFrame;
    delete mShowDlg;
	delete mAutoClose;
	delete mReceivedPath;
	delete mReceivedPathLabel;
	delete mBrowseButton;
}

void KBeamReceivePrefsDlg::addTips()
{
	QWhatsThis::add(mShowDlg, i18n("If selected, a dialog will open up \
whenever you're receiving a file. It is useful to check the status and \
progress of the transfer"));
	QWhatsThis::add(mAutoClose, i18n("If selected, the receive dialog will \
automatically close if the transfer is successful. If an error occurs, the \
dialog will not close, allowing the user to verify the error"));
	QWhatsThis::add(mReceivedPath, i18n("The path where the received files \
should be saved."));
	QWhatsThis::add(mReceivedPathLabel, i18n("The path where the received \
files should be saved."));
	QWhatsThis::add(mBrowseButton, i18n("Browse the directed where the \
received files should be saved."));
}

void KBeamReceivePrefsDlg::slotShowDlg(bool pOn)
{
	if (pOn) {
		mAutoClose->setEnabled(true);
	} else {
		mAutoClose->setEnabled(false);
	}
}

void KBeamReceivePrefsDlg::slotBrowse()
{
	QString lPath = KFileDialog::getExistingDirectory(
		mReceivedPath->text());
	if (lPath.length()) {
		mReceivedPath->setText(lPath);
	}
}
