/***************************************************************************
 *   Copyright (C) 2004 by Georges A.K.                                    *
 *   georgesak@spymac.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <klocale.h>
#include <qframe.h>
#include <qcheckbox.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

#include "kbeamsendprefsdlg.h"
#include "kbeamconfig.h"


KBeamSendPrefsDlg::KBeamSendPrefsDlg()
{
	const int lWidth = 500;
	const int lHeight = 200;

	setMinimumSize(lWidth, lHeight);

	mMainFrame = new QFrame(this);
	mMainFrame->setGeometry(0, 0, lWidth, lHeight);
	mMainFrame->setFrameShape(QFrame::Box);
	mMainFrame->setFrameShadow(QFrame::Raised);

	mShowDlg = new QCheckBox(mMainFrame, "kcfg_ShowSendDlg");
	mShowDlg->setText(
		i18n("Show progress dialog when sending"));
	mShowDlg->setGeometry(10, 10, lWidth-20, 25);
	connect(mShowDlg, SIGNAL(toggled(bool)), this, SLOT(slotShowDlg(bool)));

	mAutoClose = new QCheckBox(mMainFrame, "kcfg_AutoCloseSendDlg");
	mAutoClose->setText(
		i18n("Automatically close send dialog on success"));
	mAutoClose->setGeometry(10, 40, lWidth-20, 25);
	if (!KBeamConfig::self()->showSendDlg()) {
		mAutoClose->setEnabled(false);
	}

	// Set the tooltips and the whatsthis
	addTips();
}

KBeamSendPrefsDlg::~KBeamSendPrefsDlg()
{
    delete mMainFrame;
    delete mShowDlg;
	delete mAutoClose;
}

void KBeamSendPrefsDlg::addTips()
{
	QWhatsThis::add(mShowDlg, i18n("If selected, a dialog will open up \
whenever you're sending a file. It is useful to check the status and progress \
of the transfer"));
	QWhatsThis::add(mAutoClose, i18n("If selected, the send dialog will \
automatically close if the transfer is successful. If an error occurs, the \
dialog will not close, allowing the user to verify the error"));
}

void KBeamSendPrefsDlg::slotShowDlg(bool pOn)
{
	if (pOn) {
		mAutoClose->setEnabled(true);
	} else {
		mAutoClose->setEnabled(false);
	}
}
