/***************************************************************************
 *   Copyright (C) 2004 by Georges A.K.                                    *
 *   georgesak@spymac.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _KBEAMSYSTEM_H_
#define _KBEAMSYSTEM_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qobject.h>
#include <qthread.h>
#include <qevent.h>
#include <qguardedptr.h>
extern "C" {
#include <openobex/obex.h>
}

#include "kbeamdevicelist.h"

class KBeamObex;
class KBeamObexObject;


/** This class takes care of the infrared system. It takes care of the device
	discovery, sending and receiving. It alerts the tray icon whenever there's
	an event through KBeamEvent

 * @short Class that handles the irda system
 * @author Georges A.K. <georgesak@spymac.com>
 */
 
class KBeamSystem : public QObject
{

	Q_OBJECT

public:

	/** Constructor */
	KBeamSystem();
	/** Destructor */
	virtual ~KBeamSystem();

	/** Return the number of devices */
	int getNbDevices() const { return mDeviceList.getNbDevices(); }
	/** Returns the list of devices */
	const KBeamDeviceList &getDeviceList() const { return mDeviceList; }

	/** Initializes the system */
	bool init();

	/** Send a file */
	void sendFile(const QString pFilename);

public slots:

	/** Updates the devices in range (is called once per sec) */
	void slotDiscover();

private:

	/** Sets up the server, so that incoming requests can be handled */
	void startServer();
	/** Stops the server */
	void stopServer();
	/** Checks the settings to make sure everything is in order. For now, this
	    only checks the baud rate */
	void checksSettings();

private slots:

	/** Called when the signal for receiving a hint is emitted */
	void slotAcceptHint(KBeamObex &pObex, KBeamObexObject &pObject);

private:

	int mFd;

	QGuardedPtr<KBeamObex> mObexServer;

	KBeamDeviceList mDeviceList;

};

#endif // _KBEAMSYSTEM_H_
