/***************************************************************************
 *   Copyright (C) 2004 by Georges A.K.                                    *
 *   georgesak@spymac.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kapplication.h>
#include <kmessagebox.h>
#include <kpopupmenu.h>
#include <kaction.h>
#include <khelpmenu.h>
#include <knotifydialog.h>
#include <knotifyclient.h>
#include <kconfigskeleton.h>
#include <kconfigdialog.h>
#include <kpixmapeffect.h>
#include <kiconloader.h>
#include <klocale.h>
#include <qtooltip.h>
#include <qdir.h>
#include <qfile.h>

#include "kbeamtray.h"
#include "kbeamsystem.h"
#include "kbeamdevicedlg.h"
#include "kbeamconfig.h"
#include "kbeamevent.h"
#include "kbeamprefsdlg.h"
#include "kbeamreceiveprefsdlg.h"
#include "kbeamsendprefsdlg.h"


KBeamTray::KBeamTray()
    : KSystemTray( 0, "KBeam" ), DCOPObject("kbeamiface")
{
	// Add tooltip
	QToolTip::add(this, i18n("KBeam"));

	// Create the help menu
	mHelpMenu = new KHelpMenu(this, 
		KApplication::kApplication()->aboutData());

	// Load the icon
	mIconOn = KGlobal::iconLoader()->loadIcon(
        "kbeam", KIcon::Small, 22);
	// Make a greyed out copy
	mIconOff = mIconOn;
	KPixmapEffect::toGray(mIconOff);
	KPixmapEffect::fade(mIconOff, 0.3, QColor(255,255,255));

	// Insert the Show Devices in the context menu
	contextMenu()->insertItem(KGlobal::iconLoader()->loadIcon("info", 
		KIcon::Small, 16), i18n("S&how Devices..."), this, 
		SLOT(slotShowDevices()), NULL, 0);
	contextMenu()->insertSeparator();

	// Insert the Configuration submenu in the context menu
	KActionMenu *configActionMenu = new KActionMenu(i18n("&Configuration"),
		KGlobal::iconLoader()->loadIcon("configure", KIcon::Small, 16), 
		this, "config_menu");
	configActionMenu->plug(contextMenu());

	// Insert the Settings in the Configuration submenu
	KAction *prefsAction = new KAction(i18n("&Settings..."),
		KGlobal::iconLoader()->loadIcon("configure", KIcon::Small, 16),
		KShortcut::null(), this);
	connect(prefsAction, SIGNAL(activated()), this, SLOT(slotPreferences()));
	configActionMenu->insert(prefsAction);

	// Insert the Configure Notifications in the Configuration submenu
	KAction* notificationAction = KStdAction::configureNotifications(this,
		SLOT(slotConfigureNotifications()), actionCollection());
	configActionMenu->insert(notificationAction);

	// Insert the Help submenu in the context menu
	KActionMenu *helpActionMenu = new KActionMenu(i18n("&Help"),
		KGlobal::iconLoader()->loadIcon("help", KIcon::Small, 16), this, "help_menu");
	helpActionMenu->plug(contextMenu());

	// Insert the Report a bug in the Help submenu
	KAction* reportBugAction = KStdAction::reportBug(this,
		SLOT(slotReportBug()), actionCollection());
	helpActionMenu->insert(reportBugAction);

	// Insert seperator
	helpActionMenu->insert(new KActionSeparator(this));

	// Insert the About Dialog in the Help submenu
	KAction* aboutAction = KStdAction::aboutApp(this,
		SLOT(slotAbout()), actionCollection());
	helpActionMenu->insert(aboutAction);

	// Insert the KDE About Dialog in the Help submenu
	KAction* aboutKDEAction = KStdAction::aboutKDE(this,
		SLOT(slotAboutKDE()), actionCollection());
	helpActionMenu->insert(aboutKDEAction);

	// Create the Device List Dialog, and connect the signal
	mDeviceDlg = new KBeamDeviceDlg(this);
	connect(this, SIGNAL(signalDevicesChanged(const KBeamDeviceList&)), 
		mDeviceDlg, SLOT(slotUpdateDevices(const KBeamDeviceList& )));

	/* Create the Notifification dialog. It's done this way because we want a
	   non-Modal window */
	mNotificationsDlg = new KNotifyDialog(this, "", false);

	// If this is set to true, the application can quit (see closeEvent())
	mAllowClose = true;

	// Connect the quit selected signal
	connect(this, SIGNAL(quitSelected()), this, SLOT(slotQuitSelected()));

}

KBeamTray::~KBeamTray()
{
	disableSend();

	delete mSystem;
	delete mDeviceDlg;
	delete mHelpMenu;
	delete mNotificationsDlg;
}

bool KBeamTray::start(bool pForceIcon, bool pFirstInstance)
{
	// If forced or 2nd instance, show icon
	if (pForceIcon || !pFirstInstance) {
		KConfigSkeletonItem *lItem = 
			KBeamConfig::self()->findItem("AlwaysVisible");
		lItem->setProperty(true);
		KBeamConfig::self()->writeConfig();
		if (!pFirstInstance) {
			showIcon();
			return true;
		}
	}

	/* It's the first instance. Disable icon and show or hide it depending on
	   the configuration */

	disableIcon();
	if (KBeamConfig::alwaysVisible()) {
		showIcon();
	} else {
		hideIcon();
	}

	// Check the last version that was ran
	checkLastVersion();

	// Create the system
	mSystem = new KBeamSystem();
	// Initialise the system and return false if an error occured
	if (!mSystem->init()) {
		KMessageBox::error(0, i18n("No IRDA support was found !"), 
			i18n("Error"));
		return false;
	}

	return true;
}

void KBeamTray::dcopSendFile(const QString &pFilename)
{
	mSystem->sendFile(pFilename);
}

void KBeamTray::showIcon()
{
	show();
}

void KBeamTray::hideIcon()
{
	hide();
}

void KBeamTray::enableIcon()
{
	setPixmap(mIconOn);
}

void KBeamTray::disableIcon()
{
	setPixmap(mIconOff);
}

void KBeamTray::enableSend()
{
/* Exec=dcop kbeam kbeamiface dcopSendFile %U \n \*/
	QString lFilename = QDir::homeDirPath() + 
		"/.kde/share/apps/konqueror/servicemenus/kbeam.desktop";
	QString lText = "[Desktop Action InfraredSendFile] \n \
Exec=kbeam %U \n \
Icon=kbeam \n \
Name=Beam  \n \
[Desktop Entry] \n \
Actions=InfraredSendFile \n \
Encoding=UTF-8 \n \
Icon=kbeam \n \
ServiceTypes=all/allfiles \n \
ExcludeServiceTypes=kdedevice";

	QDir lDir;
	lDir.mkdir(QDir::homeDirPath() + 
		"/.kde/share/apps/konqueror/servicemenus");
	QFile lFile(lFilename);
	if (!lFile.open(IO_WriteOnly)) {
		KMessageBox::error(0, i18n("Couldn't add service menu !"), 
			i18n("Error"));
		return;
	}

	lFile.writeBlock(lText.ascii(), lText.length());
	lFile.flush();

	lFile.close();
}

void KBeamTray::disableSend()
{
	QString mFilename = QDir::homeDirPath() + 
		"/.kde/share/apps/konqueror/servicemenus/kbeam.desktop";

	QFile lFile(mFilename);
	lFile.remove();
}

void KBeamTray::foundDevice(QString pName)
{
	// Send user notification
	QString lText = i18n("Device %1 is in range").arg(pName);
	KNotifyClient::event(winId(), "BeamDeviceFound", lText.ascii());

	enableIcon();
	enableSend();

	show();
}

void KBeamTray::lostDevice(QString pName)
{
	// Send user notification
	QString lText = 
		i18n("Connection with %1 has been lost").arg(pName);
	KNotifyClient::event(winId(), "BeamDeviceLost", lText.ascii());

	// Disable icon and send if there are no devices in range anymore
	if (!mSystem->getNbDevices()) {
		disableIcon();
		disableSend();
	}

	if (!KBeamConfig::alwaysVisible()) {
		hide();
	}
}

void KBeamTray::mousePressEvent(QMouseEvent *pEvent)
{
	if (!rect().contains(pEvent->pos())) {
		return;
	}

	// Left click = Show Devices. Middle or right click = Context Menu
	switch (pEvent->button()) {
	case LeftButton:
		slotShowDevices();
		break;
	case MidButton:
	case RightButton:
		contextMenuAboutToShow(contextMenu());
		contextMenu()->popup(pEvent->globalPos());
		break;
	default:
		break;
	}
}

void KBeamTray::customEvent (QCustomEvent *pEvent)
{
	KBeamEvent *lEvent = (KBeamEvent *)pEvent;
	QString lText;

	/* Check which event it is, and send notification accordingly
	   See KBeamEvent for more details about event types */
	switch (lEvent->type()) {
	case KBeamEvent::ErrorMsg:
		KMessageBox::error(0, lEvent->getErrorMsg().ascii(), i18n("Error"));
		break;

	case KBeamEvent::DeviceFound:
		foundDevice(lEvent->getDevice().getName());
		emit signalDevicesChanged(mSystem->getDeviceList());
		break;
	case KBeamEvent::DeviceLost:
		lostDevice(lEvent->getDevice().getName());
		emit signalDevicesChanged(mSystem->getDeviceList());
		break;

	case KBeamEvent::StartReceive:
		lText = i18n("Receiving file : ") + lEvent->getFilename();
		KNotifyClient::event(winId(), "BeamReceivingFile", lText.ascii());
		break;
	case KBeamEvent::FileReceived:
		lText = i18n("Received file : ") + lEvent->getFilename();
		KNotifyClient::event(winId(), "BeamReceivedFile", lText.ascii());
		break;
	case KBeamEvent::ReceiveAborted:
		lText = i18n("Error while receiving file : ") + lEvent->getFilename();
		KNotifyClient::event(winId(), "BeamReceiveAborted", lText.ascii());
		break;

	case KBeamEvent::StartSend:
		lText = i18n("Sending file : ") + lEvent->getFilename();
		KNotifyClient::event(winId(), "BeamSendingFile", lText.ascii());
		break;
	case KBeamEvent::FileSent:
		lText = i18n("Sent file : ") + lEvent->getFilename();
		KNotifyClient::event(winId(), "BeamSentFile", lText.ascii());
		break;
	case KBeamEvent::SendAborted:
		lText = i18n("Error while sending file : ") + lEvent->getFilename();
		KNotifyClient::event(winId(), "BeamSendAborted", lText.ascii());
		break;

	case KBeamEvent::LinkInterrupted:
		lText = i18n("Link interrupted ! Realign the devices");
		KNotifyClient::event(winId(), "BeamLinkInterrupted", lText.ascii());
		break;
	default:
		break;
	}
}

void KBeamTray::closeEvent(QCloseEvent *pEvent)
{
	if (mAllowClose) {
		QWidget::closeEvent(pEvent);
	} else {
		pEvent->ignore();
	}
	mAllowClose = true;
}

void KBeamTray::checkLastVersion()
{
	const int lUpgradeVersion = 60;

	KConfigSkeletonItem *lItem = 
		KBeamConfig::self()->findItem("LastVersion");
	// Check if the last version used is old
	if (lItem->property().toInt() >= lUpgradeVersion) {
		return;
	}
	lItem->setProperty(lUpgradeVersion);
	KBeamConfig::self()->writeConfig();

	int lAnswer = KMessageBox::questionYesNo(0, i18n("This version of \
KBeam is running for the first time. Should the Configuration Dialog \
be opened ?"), i18n("First run"));
	if (lAnswer == KMessageBox::Yes) {
		slotPreferences();
	}
}

void KBeamTray::slotConfigureNotifications()
{
	mNotificationsDlg->show();
}

void KBeamTray::slotShowDevices()
{
	if (mDeviceDlg->isShown()) {
		mDeviceDlg->hide();
		contextMenu()->changeItem(0, i18n("&Show devices"));
	} else {
		mDeviceDlg->setInitialSize(QSize(530, 460), true);
		mDeviceDlg->show();
		contextMenu()->changeItem(0, i18n("&Hide devices"));
	}
}

void KBeamTray::slotReportBug()
{
	mHelpMenu->reportBug();
}

void KBeamTray::slotAbout()
{
	mHelpMenu->aboutApplication();
}

void KBeamTray::slotAboutKDE()
{
	mHelpMenu->aboutKDE();
}

void KBeamTray::slotPreferences()
{
	if (KConfigDialog::showDialog("prefs")) {
		return;
	}
	KConfigDialog* lConfigDialog = new KConfigDialog(NULL, "settings", 
		KBeamConfig::self());
	KBeamPrefsDlg *lPrefsDlg = new KBeamPrefsDlg();
	lConfigDialog->addPage(lPrefsDlg, i18n("General"), "configure");
	KBeamReceivePrefsDlg *lReceivePrefsDlg = new KBeamReceivePrefsDlg();
	lConfigDialog->addPage(lReceivePrefsDlg, i18n("Receive"), "configure");
	KBeamSendPrefsDlg *lSendPrefsDlg = new KBeamSendPrefsDlg();
	lConfigDialog->addPage(lSendPrefsDlg, i18n("Send"), "configure");

	connect(lConfigDialog, SIGNAL(settingsChanged()), 
		this, SLOT(slotUpdateConfiguration())); 
	connect(lConfigDialog, SIGNAL(finished()), this, 
		SLOT(slotUpdateConfiguration())); 
	lConfigDialog->show();
}

void KBeamTray::slotUpdateConfiguration()
{
	if (KBeamConfig::alwaysVisible() && isHidden()) {
		show();
	} else if (!KBeamConfig::alwaysVisible() && isShown() && 
		mSystem->getNbDevices() == 0) {
		hide();
	}

	// If the path does not exist, show error and set to default
	QDir lDir(KBeamConfig::receivedPath());
	if (!lDir.exists()) {
		KMessageBox::error(0, 
			i18n("The received path does not exist, \
reverting to default path."), i18n("Error"));
		KConfigSkeletonItem *lItem = 
			KBeamConfig::self()->findItem("ReceivedPath");
		lItem->setDefault();
		KBeamConfig::self()->writeConfig();
	}
}

void KBeamTray::slotQuitSelected()
{
	int lAutoStart = KMessageBox::questionYesNoCancel( 0,
		i18n("Should KBeam be started automatically next time you login ?"),
		i18n("Automatically start KBeam ?") );

	mAllowClose = true;

	KConfigSkeletonItem *lItem = 
		KBeamConfig::self()->findItem("AutoStart");

	switch (lAutoStart) {
	case KMessageBox::Yes:
		lItem->setProperty(true);
		KBeamConfig::self()->writeConfig();
		break;
	case KMessageBox::No:
		lItem->setProperty(false);
		KBeamConfig::self()->writeConfig();
		break;
	default:
		mAllowClose = false;
	}
}
