/***************************************************************************
 *   Copyright (C) 2004 by Georges A.K.                                    *
 *   georgesak@spymac.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _KBEAMTRAY_H_
#define _KBEAMTRAY_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <ksystemtray.h>
#include <kpixmap.h>
#include <qguardedptr.h>
#include <qstring.h>
#include <dcopobject.h>


class KNotifyDialog;
class QMouseEvent;
class QCustomEvent;
class KHelpMenu;
class KBeamSystem;
class KBeamDeviceDlg;
class KBeamDeviceList;


/** This is the main widget of the application. It takes care of the system
    tray icon. It also takes care of all the dialogs (config, devicelist...)
 * @short The Tray Icon
 * @author Georges A.K. <georgesak@spymac.com>
 */
class KBeamTray : public KSystemTray, public DCOPObject
{
	Q_OBJECT
	K_DCOP

public:

	/** Constructor */
	KBeamTray();
	/** Destructor */
	~KBeamTray();

	/** This starts the tray icon. If pForceIcon is true, The icon is shown
	    regardless of the user's preferences, and we actually change that 
		preference. If pFirstInstance is false, an instance of KBeam is already
		running. Returns false if an error occured, else true */
	bool start(bool pForceIcon, bool pFirstInstance);

k_dcop:

	/** This is a dcop call to send a file (used in the action menu).
	    Takes the filename as a parameter */
	void dcopSendFile(const QString &pFilename);

signals:

	/** This signal is emitted whenever there are device changes */
	void signalDevicesChanged(const KBeamDeviceList&);

private:

	/** Shows the system tray icon */
	void showIcon();
	/** Hides the system tray icon */
	void hideIcon();

	/** Enables the system tray icon (becomes coloured) */
	void enableIcon();
	/** Disables the system tray icon (becomes grayed out) */
	void disableIcon();

	/** Enables sending files through the action menu of Konqueror */
	void enableSend();
	/** Disables sending files through the action menu of Konqueror */
	void disableSend();

	/** This is called when a device is found, and it alerts the user.
	    Takes the name of the device as argument */
	void foundDevice(QString pName);
	/** This is called when a device is lost, and it alerts the user.
	    Takes the name of the device as argument */
	void lostDevice(QString pName);

	/** This is reimplemented so that clicking the icon would open up the
	    device list (right click is still context menu) */
	void mousePressEvent(QMouseEvent *pEvent);
	/** Handles all the custom events. See class KBeamEvent for more details */
	void customEvent (QCustomEvent *pEvent);
	/** Reimplemented since the user can cancel the closing of the app */
	void closeEvent(QCloseEvent *pEvent);

	/** Checks the last version of KBeam that was ran. Useful for first time
	    notifications */
	void checkLastVersion();

private slots:

	/** Slot that is called when the configuration changes. It puts them into
	    effect */
	void slotUpdateConfiguration();
	/** Slot that opens up the device list dialog */
	void slotShowDevices();
	/** Slot that opens up the configuration dialog */
	void slotPreferences();
	/** Slot that opens up the configuration of the notifications dialog */
	void slotConfigureNotifications();
	/** Slot that opens up the "report a bug" dialog */
	void slotReportBug();
	/** Slot that open up the about dialog */
	void slotAbout();
	/** Slot that open up the KDE about dialog */
	void slotAboutKDE();
	/** Slot that is called when the user quits the application */
	void slotQuitSelected();

private:

	bool mAllowClose;

	KPixmap mIconOn;
	KPixmap mIconOff;

	QGuardedPtr<KBeamSystem> mSystem;
	QGuardedPtr<KBeamDeviceDlg> mDeviceDlg;

	QGuardedPtr<KHelpMenu> mHelpMenu;
	QGuardedPtr<KNotifyDialog> mNotificationsDlg;
};

#endif // _KBEAMTRAY_H_
